"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DrupalJsonApiParams = void 0;
const qs = require("qs");
class DrupalJsonApiParams {
    constructor() {
        this.data = {
            filter: {},
            sort: [],
            include: [],
            page: undefined,
            fields: {},
        };
    }
    addFields(type, fields) {
        this.data.fields[type] = fields.join(',');
        return this;
    }
    addSort(path, direction) {
        let prefix = '';
        if (direction !== undefined && direction === 'DESC') {
            prefix = '-';
        }
        this.data.sort = this.data.sort.concat(prefix + path);
        return this;
    }
    addPageLimit(limit) {
        this.data.page = { limit };
        return this;
    }
    addInclude(fields) {
        this.data.include = this.data.include.concat(fields);
        return this;
    }
    addGroup(name, conjunction = 'OR', memberOf) {
        this.data.filter[name] = {
            group: Object.assign({ conjunction }, (memberOf !== undefined && { memberOf })),
        };
        return this;
    }
    addFilter(path, value, operator = '=', memberOf) {
        const name = this.getIndexId(this.data.filter, path);
        // Allow null values only for IS NULL and IS NOT NULL operators.
        if (value === null) {
            if (!(operator === 'IS NULL' || operator === 'IS NOT NULL')) {
                throw new TypeError(`Value cannot be null for the operator "${operator}"`);
            }
            this.data.filter[name] = {
                condition: Object.assign(Object.assign({ path }, { operator }), (memberOf !== undefined && { memberOf })),
            };
            return this;
        }
        if (Array.isArray(value)) {
            switch (operator) {
                case 'BETWEEN':
                case 'NOT BETWEEN':
                    if (value.length !== 2) {
                        throw new TypeError(`Value must consists of 2 items for the "${operator}"`);
                    }
                    break;
                case 'IN':
                case 'NOT IN':
                    break;
                default:
                    throw new TypeError(`Value cannot be an array for the operator "${operator}"`);
            }
            this.data.filter[name] = {
                condition: Object.assign(Object.assign({ path,
                    value }, { operator }), (memberOf !== undefined && { memberOf })),
            };
            return this;
        }
        // Validate filter
        if (operator === '=' && memberOf === undefined && this.data.filter[path] === undefined) {
            this.data.filter[path] = value;
            return this;
        }
        this.data.filter[name] = {
            condition: Object.assign(Object.assign({ path,
                value }, (operator !== '=' && { operator })), (memberOf !== undefined && { memberOf })),
        };
        return this;
    }
    getIndexId(obj, proposedKey) {
        let key;
        if (obj[proposedKey] === undefined) {
            key = proposedKey;
        }
        else {
            key = Object.keys(obj).length.toString();
        }
        return key;
    }
    getQueryObject() {
        const data = Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, (this.data.filter !== {} && { filter: this.data.filter })), (!!this.data.include.length && { include: this.data.include.join(',') })), (this.data.page !== undefined && { page: this.data.page })), (!!this.data.sort.length && { sort: this.data.sort.join(',') })), (this.data.fields !== {} && { fields: this.data.fields }));
        return data;
    }
    getQueryString(options) {
        const data = this.getQueryObject();
        return qs.stringify(data, options);
    }
    clear() {
        this.data = {
            filter: {},
            sort: [],
            include: [],
            page: undefined,
            fields: {},
        };
        return this;
    }
    initializeWithQueryObject(input) {
        this.clear();
        if (input.filter !== undefined) {
            this.data.filter = input.filter;
        }
        if (input.include !== undefined) {
            this.data.include = input.include.split(',');
        }
        if (input.page !== undefined) {
            this.data.page = input.page;
        }
        if (input.sort !== undefined) {
            this.data.sort = input.sort.split(',');
        }
        if (input.fields !== undefined) {
            this.data.fields = input.fields;
        }
        return this;
    }
    initializeWithQueryString(input) {
        this.clear();
        this.initializeWithQueryObject(qs.parse(input));
        return this;
    }
    clone(input) {
        const data = JSON.parse(JSON.stringify(input.getQueryObject()));
        this.initializeWithQueryObject(data);
        return this;
    }
    initialize(input) {
        if (typeof input === 'object') {
            try {
                // if the input has getQueryObject() we attempt to clone.
                input.getQueryObject();
                this.clone(input);
            }
            catch (error) {
                // In any case if cloning failed, we attempt to initialize
                // with query object.
                this.initializeWithQueryObject(input);
            }
        }
        else {
            this.initializeWithQueryString(input);
        }
        return this;
    }
}
exports.DrupalJsonApiParams = DrupalJsonApiParams;
